# Getting Started

## Structure

The SDK contains headers that you need for building a plugin that can interact and modify the UI of CapturyLive or CapturyStudio. It also contains some sample plugins.

Please note that there are two other APIs available for interacting with CapturyLive / CapturyStudio:
* There is a Python automation API that allows you to start CapturyLive / CapturyStudio in a headless mode. This is primarily used for integration with third-party applications that take care of camera control and calibration like Vicon Nexus or Qualisys QTM or for automating tracking tasks. This is documented [here](http://doc.captury.com/python.html).
* We provide a streaming plugin for CapturyLive that comes with a client library that can easily be integrated into software that provides a plugin interface like popular game engines or 3D applications. The client plugin `RemoteCaptury` can be found on [github](https://github.com/thecaptury/RemoteCaptury).

Use this SDK if you want to add a custom panel to the user interface of CapturyLive but not replace it entirely. In most other cases the remote control interface `RemoteCaptury` is probably the right choice.
For replacing the interface entirely build your interface with a game engine and use `RemoteCaptury` for communication with CapturyLive.

## Prepare

install `cmake`, `gcc/clang` (incl. C++ support) and optionally `qt5base-dev`

## Build

```
tar -xf CapturyLiveSDK*.zip
cd CapturyLiveSDK
mkdir build && cd build
cmake ../plugins
cmake --build .
```
Then copy the lib*Plugin.so into the folder where CapturyLive is (the bin folder). Next time CapturyLive starts it should start these plugins as well.

## Hack

To build your own plugin you just need to build a library that is named `*Plugin.so` / `*Plugin.dll` that provides a function `initPlugin` and is placed in the folder where the CapturyLive / CapturyStudio executable sits.

```
// provided by the plugin
// fill info structure and return 1 if successful
CAPTURY_DLL_EXPORT unsigned char initPlugin(pluginInfo* info);
```

`info` is passed to the plugin. The host application expects that the plugin fills in the details that it is interested in. In particular, some callback functions can be registered. The declaration and all functions that are provided by the host application are declared in `plugin.h`.

Note that there are two ways to access the host functionality. The function `liveGetHostSuite` allows the plugin to retrieve sets of related function pointers for example for logging, which can then be used to push a message into the logging system of the host application. Typically this would be used like so:

```
static TCLogHostSuite* logging;
unsigned char initPlugin(pluginInfo* info)
{
	logging = (TCLogHostSuite*)info->getHostSuite("Log");

	logging->info("hello world!");
}
```

This is the preferred method of accessing host functionality. Unfortunately, not all functions have been ported yet. So the old API is still available and required for some plugins. There are a lot of functions at the bottom of the header. Please let us know if any functionality you need is missing. We'll consider adding it.
